use std::fs;
use std::io::Read;
use std::process::Command;
use std::process::Stdio;

use super::Executor;

use super::helper::*;

#[derive(Debug, Clone, Copy)]
pub struct PythonExecutor {}

impl Executor for PythonExecutor {
    fn id(&self) -> String {
        "python".into()
    }

    fn exec(&self, code: String) -> Result<String, String> {
        assure_dir_exists().map_err(|x| x.to_string())?;

        let path = create_path();

        // 1. Save the code
        fs::write(path.clone(), code).map_err(|x| x.to_string())?;

        // 2. Run
        let mut out = Command::new("python")
            .arg(path.clone())
            .stdout(Stdio::piped())
            .spawn()
            .unwrap();

        // 3. Grab the stuff and exit
        let exit_status = out.wait().map_err(|x| x.to_string())?;

        let mut stdout: String = "".into();

        if let Some(mut sout) = out.stdout {
            let mut buf = vec![];
            sout.read_to_end(&mut buf).map_err(|x| x.to_string())?;
            stdout += String::from_utf8(buf).map_err(|x| x.to_string())?.as_str();
        } else {
            stdout += "Couldn't get stdout from the process"
        }

        stdout += "\n\n";
        stdout += exit_code_msg(exit_status).as_str();

        Ok(stdout)
    }
}